local slots = {
	"HeadSlot",
	"ShoulderSlot",
	"BackSlot",
	"ChestSlot",
	"WristSlot",
	"HandsSlot",
	"WaistSlot",
	"LegsSlot",
	"FeetSlot",
	"MainHandSlot",
	"SecondaryHandSlot",
}

local currentVoidItem = 0

local origClickVoidStorageSlot = ClickVoidStorageSlot

function ClickVoidStorageSlot(slot, isRightClick)
	local _, _, wasLocked = GetVoidItemInfo(slot)
	origClickVoidStorageSlot(slot, isRightClick)
	if not wasLocked and select(3, GetVoidItemInfo(slot)) and not isRightClick then
		currentVoidItem = slot
	elseif not select(3, GetVoidItemInfo(currentVoidItem)) then
		currentVoidItem = 0
	end
end

local origClickTransmogrifySlot = ClickTransmogrifySlot

function ClickTransmogrifySlot(slotID)
	local itemID, _, locked = GetVoidItemInfo(currentVoidItem)
	if locked then
		local soundCVar = GetCVar("Sound_EnableSFX")
		SetCVar("Sound_EnableSFX", 0)
		UseVoidItemForTransmogrify(currentVoidItem, slotID)
		SetCVar("Sound_EnableSFX", soundCVar)
		local equipped = GetInventoryItemID("player", slotID)
		if equipped and CanTransmogrifyItemWithItem(itemID, equipped) then
			ClearCursor()
			currentVoidItem = 0
		end
		return
	end
	local _, _, _, hasPending = GetTransmogrifySlotInfo(slotID)
	origClickTransmogrifySlot(slotID)
	if hasPending and not GetCursorInfo() then
		local soundCVar = GetCVar("Sound_EnableSFX")
		SetCVar("Sound_EnableSFX", 0)
		ClearCursor()
		SetCVar("Sound_EnableSFX", soundCVar)
	end
end

local function openTransmog()
	ItemAlteration_LoadUI()
	TransmogrifyFrame_Show()
end

SlashCmdList["VOIDTRANSMOG"] = openTransmog
SLASH_VOIDTRANSMOG1 = "/transmog"
SLASH_VOIDTRANSMOG2 = "/vt"

local function createButton(name, parent)
	local button = CreateFrame("Button", name, parent, "UIPanelButtonTemplate")
	button:SetWidth(100)
	button:SetText("Transmogrify")
	button:SetScript("OnClick", openTransmog)
	return button
end

local button = createButton("BankTransmogrifyButton", BankFrame)
button:SetPoint("RIGHT", BankItemSearchBox, "LEFT", -16, 0)

local isAtTransmogrifier

button:RegisterEvent("ADDON_LOADED")
button:RegisterEvent("TRANSMOGRIFY_OPEN")
button:RegisterEvent("TRANSMOGRIFY_CLOSE")
button:RegisterEvent("VOID_STORAGE_OPEN")
button:SetScript("OnEvent", function(self, event, addon)
	if event == "TRANSMOGRIFY_OPEN" then
		isAtTransmogrifier = true
		TransmogrifyFrame_UpdateApplyButton()
	elseif event == "TRANSMOGRIFY_CLOSE" then
		isAtTransmogrifier = false
	elseif event == "VOID_STORAGE_OPEN" then
		local button = createButton("VoidTransmogrifyButton", VoidStorageFrame)
		button:SetPoint("RIGHT", VoidItemSearchBox, "LEFT", -16, 0)
		self:UnregisterEvent(event)
	elseif event == "ADDON_LOADED" and addon == "Blizzard_ItemAlterationUI" then
		-- apparently pending items are retained between sessions, so clear them on load
		local soundCVar = GetCVar("Sound_EnableSFX")
		SetCVar("Sound_EnableSFX", 0)
		for i, slot in ipairs(slots) do
			ClearTransmogrifySlot(GetInventorySlotInfo(slot))
		end
		SetCVar("Sound_EnableSFX", soundCVar)
		
		UIPanelWindows["TransmogrifyFrame"].pushable = 1
		hooksecurefunc("TransmogrifyFrame_UpdateApplyButton", function()
			if not isAtTransmogrifier then
				TransmogrifyApplyButton:Disable()
			end
		end)
		TransmogrifyApplyButton:SetScript("OnEnter", function(self)
			if not isAtTransmogrifier then
				GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
				GameTooltip:AddLine("Visit transmogrifier to apply changes.")
				GameTooltip:Show()
			end
		end)
		TransmogrifyApplyButton:SetScript("OnLeave", GameTooltip_Hide)
		TransmogrifyApplyButton:SetMotionScriptsWhileDisabled(true)
		self:UnregisterEvent(event)
	end
end)